@@ -15,7 +15,7 @@ from tornado.web import Application
 from torext import settings
 from torext import errors
 from torext.testing import TestClient, AppTestCase
-from torext.log import set_logger, set_nose_formatter
+from torext.log import set_nose_formatter, app_log, request_log
 from torext.route import Router
 from torext.utils import json_encode, json_decode
 
@@ -46,6 +46,8 @@ class TorextApp(object):
             GZIP
             XHEADERS
         """
+        global settings
+
         if settings_module:
             self.module_config(settings_module)
         if extra_settings:
@@ -65,19 +67,31 @@ class TorextApp(object):
         self.json_encoder = json_encode
         self.json_decoder = json_decode
 
-        global settings
         self.settings = settings
 
         TorextApp.current_app = self
 
-        # call ``setup``
-        print 'Initialize finished, setup torext the first time..'
-        self.setup()
-
-    def update_settings(self, incoming):
+    def update_settings(self, incoming, convert_type=False, log_changes=False):
         global settings
+
+        def _log(s):
+            if log_changes:
+                print s
+
         for i in incoming:
-            settings[i] = incoming[i]
+            incoming_v = incoming[i]
+
+            if i in settings:
+                _log('Settings update "%s": %s -> %s' % (i, settings[i], incoming_v))
+
+                if convert_type:
+                    incoming_v = _convert_type(incoming_v, settings[i])
+            else:
+                _log('Settings add "%s": %s' % (i, incoming_v))
+
+            settings[i] = incoming_v
+
+        self.is_setuped = False
 
     def set_root_path(self, root_path=None, settings_module=None):
         """
@@ -127,9 +141,14 @@ class TorextApp(object):
             'ui_modules',
             'static_handler_class',
             'static_handler_args',
+
+            'autoreload',
+            'compiled_template_cache',
+            'static_hash_cache',
+            'serve_traceback'
         ]
         options = {
-            'log_function': _log_function,
+            'log_function': self._log_function,
             'ui_modules': self.uimodules,
         }
 
@@ -158,7 +177,7 @@ class TorextApp(object):
                 if not os.path.isabs(v):
                     v = os.path.abspath(
                         os.path.join(self.root_path, v))
-                    logging.debug('Fix %s to be absolute: %s' % (k, v))
+                    app_log.debug('Fix %s to be absolute: %s' % (k, v))
                 options[k] = v
 
     def _get_handlers_on_host(self, host=None):
@@ -205,11 +224,10 @@ class TorextApp(object):
         """
         Optional function
         """
-        assert hasattr(settings_module, '__file__'), 'settings passed in initialize() must be a module'
+        assert hasattr(settings_module, '__file__'), 'settings must be a module'
         # set root_path according to module file
         self.set_root_path(settings_module=settings_module)
-        #logging.info('set root_path: %s', self.root_path)
-        print 'Set root_path: %s' % self.root_path
+        app_log.debug('Set root_path: %s', self.root_path)
 
         global settings
 
@@ -233,6 +251,8 @@ class TorextApp(object):
 
         example:
         $ python app.py --PORT=1000
+
+        NOTE This method is deprecated, use `torext.script` to parse command line arguments instead.
         """
 
         args = sys.argv[1:]
@@ -256,7 +276,7 @@ class TorextApp(object):
                 new_keys.append(key)
 
         if existed_keys:
-            print 'Changed settings:'
+            app_log.debug('Changed settings:')
             for i in existed_keys:
                 before = settings[i]
                 type_ = type(before)
@@ -270,35 +290,27 @@ class TorextApp(object):
                 else:
                     _value = type_(args_dict[i])
                 settings[i] = _value
-                print '  %s  [%s]%s (%s)' % (i, type(settings[i]), settings[i], before)
+                app_log.debug('  %s  [%s]%s (%s)', i, type(settings[i]), settings[i], before)
 
         if new_keys:
-            print 'New settings:'
+            app_log.debug('New settings:')
             for i in new_keys:
                 settings[i] = args_dict[i]
-                print '  %s  %s' % (i, args_dict[i])
+                app_log.debug('  %s  %s', i, args_dict[i])
 
-        # NOTE if ``command_line_config`` is called, torext must be re-setup
-        print 'Command line config finished, re-setup torext..'
-        self.setup()
+        # NOTE if ``command_line_config`` is called, logging must be re-configed
+        self.update_settings({})
 
     def setup(self):
-        """
-        setups before run
+        """This function will be called both before `run` and testing started.
         """
         testing = settings.get('TESTING')
 
-        if not testing:
-            print 'Setup torext..'
-
-        # setup root logger (as early as possible)
-        logging_kwargs = settings['LOGGING_OPTIONS'].copy()
-        logging_kwargs['level'] = settings['LOGGING']
-        print 'logging kwargs: %s' % logging_kwargs
-        set_logger('', **logging_kwargs)
         if testing:
-            print 'testing, set nose formatter', logging_kwargs
-            set_nose_formatter(logging_kwargs)
+            # Fix nose handler in testing situation.
+            config = settings['LOGGERS'].get('', {})
+            set_nose_formatter(config)
+            print 'testing, set nose formatter: %s' % config
 
         # reset timezone
         os.environ['TZ'] = settings['TIME_ZONE']
@@ -321,12 +333,10 @@ class TorextApp(object):
                 parent_path = os.path.dirname(self.root_path)
                 if not _abs(parent_path) in [_abs(i) for i in sys.path]:
                     sys.path.insert(0, parent_path)
-                    if not testing:
-                        logging.info('Add %s to sys.path' % _abs(parent_path))
+                    app_log.info('Add %s to sys.path' % _abs(parent_path))
             try:
                 __import__(settings['PROJECT'])
-                if not testing:
-                    logging.debug('import package `%s` success' % settings['PROJECT'])
+                app_log.debug('import package `%s` success' % settings['PROJECT'])
             except ImportError:
                 raise ImportError('PROJECT could not be imported, may be app.py is outside the project'
                                   'or there is no __init__ in the package.')
@@ -334,6 +344,13 @@ class TorextApp(object):
         self.is_setuped = True
 
     def _init_infrastructures(self):
+        multiprocessing = False
+        if settings['PROCESSES'] and settings['PROCESSES'] > 1:
+            if settings['DEBUG']:
+                app_log.info('Multiprocess could not be used in debug mode')
+            else:
+                multiprocessing = True
+
         if self.io_loop:
             if not self.io_loop.initialized():
                 # this means self.io_loop is a customized io_loop, so `install` should be called
@@ -341,7 +358,9 @@ class TorextApp(object):
                 #print self.io_loop.__class__.__name__
                 self.io_loop.install()
         else:
-            self.io_loop = IOLoop.instance()
+            # NOTE To support running tornado for multiple processes, we do not instance ioloop if multiprocessing is True
+            if not multiprocessing:
+                self.io_loop = IOLoop.instance()
 
         application = self._make_application()
 
@@ -350,21 +369,22 @@ class TorextApp(object):
         listen_kwargs = {}
         if settings.get('ADDRESS'):
             listen_kwargs['address'] = settings.get('ADDRESS')
-        if not settings['TESTING'] and settings['DEBUG']:
-            if settings['PROCESSES'] and settings['PROCESSES'] > 1:
-                logging.info('Multiprocess could not be used in debug mode')
-            try:
-                http_server.listen(settings['PORT'], **listen_kwargs)
-            except socket.error, e:
-                logging.warning('socket.error detected on http_server.listen, set ADDRESS="0.0.0.0" in settings to avoid this problem')
-                raise e
-        else:
+
+        if multiprocessing:
+            # Multiprocessing mode
             try:
                 http_server.bind(settings['PORT'], **listen_kwargs)
             except socket.error, e:
-                logging.warning('socket.error detected on http_server.listen, set ADDRESS="0.0.0.0" in settings to avoid this problem')
+                app_log.warning('socket.error detected on http_server.listen, set ADDRESS="0.0.0.0" in settings to avoid this problem')
                 raise e
             http_server.start(settings['PROCESSES'])
+        else:
+            # Single process mode
+            try:
+                http_server.listen(settings['PORT'], **listen_kwargs)
+            except socket.error, e:
+                app_log.warning('socket.error detected on http_server.listen, set ADDRESS="0.0.0.0" in settings to avoid this problem')
+                raise e
 
         self.http_server = http_server
         self.application = application
@@ -375,6 +395,10 @@ class TorextApp(object):
             return self.io_loop._running
         return False
 
+    def _instance_ioloop(self):
+        if not self.io_loop:
+            self.io_loop = IOLoop.instance()
+
     def run(self):
         if not self.is_setuped:
             self.setup()
@@ -385,6 +409,7 @@ class TorextApp(object):
             self.log_app_info()
 
         try:
+            self._instance_ioloop()
             self.io_loop.start()
         except KeyboardInterrupt:
             print '\nStopping ioloop.. ',
@@ -395,12 +420,19 @@ class TorextApp(object):
     def log_app_info(self):
         mode = settings['DEBUG'] and 'Debug' or 'Product'
         content = '\nMode %s, Service Info:' % mode
+        loggers_info = {}
+        for k in settings['LOGGERS']:
+            _logger = logging.getLogger(k)
+            #loggers_info[k] = {i: getattr(_logger, i) for i in ('level', 'handlers', 'propagate')}
+            loggers_info[k] = dict((i, getattr(_logger, i)) for i in ('level', 'handlers', 'propagate'))
+            level = loggers_info[k]['level']
+            loggers_info[k]['level'] = '%s (%s)' % (level, logging._levelNames[level])
 
         info = {
             'Project': settings['PROJECT'] or 'None (better be assigned)',
             'Port': settings['PORT'],
             'Processes': settings['DEBUG'] and 1 or settings['PROCESSES'],
-            'Logging(root) Level': settings['LOGGING'],
+            'Loggers': loggers_info,
             'Locale': settings['LOCALE'],
             'Debug': settings['DEBUG'],
             'Home': 'http://127.0.0.1:%s' % settings['PORT'],
@@ -415,10 +447,10 @@ class TorextApp(object):
         info['URL Patterns(by sequence)'] = '\n    ' + '\n    '.join(buf)
 
         for k in ['Project', 'Port', 'Processes',
-                  'Logging(root) Level', 'Locale', 'Debug', 'Home', 'URL Patterns(by sequence)']:
+                  'Loggers', 'Locale', 'Debug', 'Home', 'URL Patterns(by sequence)']:
             content += '\n- %s: %s' % (k, info[k])
 
-        logging.info(content)
+        app_log.info(content)
 
     def test_client(self, **kwgs):
         return TestClient(self, **kwgs)
@@ -447,7 +479,7 @@ class TorextApp(object):
 
     def _make_application(self, application_class=Application):
         options = self.get_application_options()
-        logging.debug('%s settings: %s', application_class.__name__, options)
+        app_log.debug('%s settings: %s', application_class.__name__, options)
 
         # this method intended to be able to called for multiple times,
         # so attributes should not be changed, just make a copy
@@ -470,25 +502,23 @@ class TorextApp(object):
         from tornado.wsgi import WSGIApplication
         return self._make_application(application_class=WSGIApplication)
 
+    def _log_function(self, handler):
+        """Override Applicaion.log_function so that what to log can be controlled.
+        """
+        if handler.get_status() < 400:
+            log_method = request_log.info
+        elif handler.get_status() < 500:
+            log_method = request_log.warning
+        else:
+            log_method = request_log.error
+        for i in settings['LOGGING_IGNORE_URLS']:
+            if handler.request.uri.startswith(i):
+                log_method = request_log.debug
+                break
 
-def _log_function(handler):
-    """
-    override Applicaion.log_function so that what to log can be controled.
-    """
-    if handler.get_status() < 400:
-        log_method = logging.info
-    elif handler.get_status() < 500:
-        log_method = logging.warning
-    else:
-        log_method = logging.error
-    for i in settings['LOGGING_IGNORE_URLS']:
-        if handler.request.uri.startswith(i):
-            log_method = logging.debug
-            break
-
-    request_time = 1000.0 * handler.request.request_time()
-    log_method("%d %s %.2fms", handler.get_status(),
-               handler._request_summary(), request_time)
+        request_time = 1000.0 * handler.request.request_time()
+        log_method("%d %s %.2fms", handler.get_status(),
+                   handler._request_summary(), request_time)
 
 
 _caller_path = None
@@ -510,3 +540,17 @@ def _guess_caller():
 
 def guess():
     return _guess_caller()
+
+
+def _convert_type(raw, before):
+    type_ = type(before)
+    if type_ is bool:
+        if raw == 'True':
+            value = True
+        elif raw == 'False':
+            value = False
+        else:
+            raise errors.ArgsParseError('Should only be True or False, got: %s' % raw)
+    else:
+        value = type_(raw)
+    return value
