@@ -24,11 +24,13 @@ DEBUG = True
 
 TESTING = False
 
-LOGGING = 'INFO'
-
-LOGGING_OPTIONS = {
-    'propagate': 1,
-    'color': True,
+LOGGERS = {
+    '': {
+        'level': 'INFO',
+        'fmt': '%(color)s[%(fixed_levelname)s %(asctime)s %(module)s:%(lineno)d]%(end_color)s %(message)s',
+        #'fmt': '[%(fixed_levelname)s %(asctime)s %(module)s:%(lineno)d] %(message)s',
+        'datefmt': '%Y-%m-%d %H:%M:%S',
+    }
 }
 
 LOG_REQUEST = False
